%
% Testing for Structural Change in the Real Growth rate of 
% per capita consumption of non-durables and services
% This code computes the Wald, LM, LR statistics for structural
% change in the parameters of a AR(1) process for consumption
% growth.
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Restricted model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 1: Importing Data and setting the number of observations.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear -exclusive rootpath;

data=dlmread(strcat(rootpath,'Output\s011.consumption_final.csv'), ",");
[T N]= size(data);
data = data(2:T-1,:);

gc=data(:,14);
[T N]=size(gc);
y=gc(2:T);

for t=1:T-1
    x(:,t)=[1; gc(t,1)];
end
[T N]=size(x');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 2:   i.  Construction of sample matrices, Sxx and Sxy.
%           ii. Two Step Efficient GMM procedure
%                   1. Estimation of S;
%                   2. Optimal Weighting Matrix, W.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% i.  Construction of sample matrices, Sxx and Sxy.
Sxx=zeros(2,2);
Sxy=zeros(2,1);
for t=1:T
    Sxx=Sxx+x(:,t)*x(:,t)';
    Sxy=Sxy+y(t,1)*x(:,t);    
end
Sxx=1/T*Sxx;
Sxy=1/T*Sxy;

% ii.   Two Step Efficient GMM procedure
W=inv(Sxx);% Inital choice of W.
b=inv(Sxx'*W*Sxx)*Sxx'*W*Sxy;% GMM estimator for W equal to the inverse of Sxx.

% 1. Estimating S 
% OBS.: S=E[error_{t}x_{t}x_{t}'] see eq. 3.5.10, pg 212
S=0;
for t=1:T
    e(t,1)=y(t)-x(:,t)'*b;
    S=S+(e(t)^(2))*x(:,t)*x(:,t)';
end
S=1/T*S;

% 2. Optimal Weighting Matrix, W
clear e b 
W=inv(S);
b_r=inv(Sxx'*W*Sxx)*Sxx'*W*Sxy;
Avar_r=inv(Sxx'*W*Sxx)*Sxx'*W*S*W*Sxx*inv(Sxx'*W*Sxx);
tstat_r=(T^(1/2))*b_r./(diag(Avar_r).^(1/2));

for t=1:T
    yhat(t)=x(:,t)'*b_r;
    e(t,1)=y(t)-yhat(t);
end
SSRr=e'*e;
SSy_r=sum((y-mean(y)).^2);

std_r=(SSRr/T)^(1/2);
R2_r=1-SSRr/SSy_r;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Restricted model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 1: Importing Data and setting the number of observations.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear x Sxx Sxy S W e N T y t  

[T N]=size(gc);
y(:,1,1)=gc(2:T);

k=11;
T=length(y);
for t=1:T-2*k+1
    bk(t)=t+k-1; % Pre-break observations
    lambda(t)=bk(t)/T;
end

for j=1:T-2*k+1
    for t=1:T
        if t<=bk(j)
            x(:,t,j)=[1; gc(t,1); 0; 0];
        else
            x(:,t,j)=[ 0; 0; 1; gc(t,1)];
        end
    end
    [T(j) N(j)]=size(x(:,:,j)');
    % i.  Construction of sample matrices, Sxx and Sxy.
    Sxx(:,:,j)=zeros(N(j),N(j),1);
    Sxy(:,:,j)=zeros(N(j),1,1);
    for t=1:T(j)
        Sxx(:,:,j)=Sxx(:,:,j)+x(:,t,j)*x(:,t,j)';
        Sxy(:,:,j)=Sxy(:,:,j)+y(t,1)*x(:,t,j);    
    end
    Sxx(:,:,j)=1/T(j)*Sxx(:,:,j);
    Sxy(:,:,j)=1/T(j)*Sxy(:,:,j);
    % ii.   Two Step Efficient GMM procedure
    W(:,:,j)=inv(Sxx(:,:,j));% Initial choice of W.
    b(:,:,j)=inv(Sxx(:,:,j)'*W(:,:,j)*Sxx(:,:,j))*Sxx(:,:,j)'*W(:,:,j)*Sxy(:,:,j);% GMM estimator for W equal to the inverse of Sxx.
    % 1. Estimating S 
    % OBS.: S=E[error^{2}_{t}x_{t}x_{t}'] see eq. 3.5.10, pg 212
    S(:,:,j)=zeros(N(j),N(j));
    for t=1:T(j)
        e(t,1,j)=y(t,1,1)-x(:,t,j)'*b(:,:,j);
        S(:,:,j)=S(:,:,j)+(e(t,1,j)^(2))*x(:,t,j)*x(:,t,j)';
    end
    % 2. Optimal Weighting Matrix, W
    W(:,:,j)=inv(S(:,:,j));
    b(:,:,j)=inv(Sxx(:,:,j)'*W(:,:,j)*Sxx(:,:,j))*Sxx(:,:,j)'*W(:,:,j)*Sxy(:,:,j);
    for t=1:T(j)
        e(t,1,j)=y(t,1,1)-x(:,t,j)'*b(:,:,j);
        S(:,:,j)=S(:,:,j)+(e(t,1,j)^(2))*x(:,t,j)*x(:,t,j)';
    end
    S(:,:,j)=1/T(j)*S(:,:,j);
    Avar(:,:,j)=inv(Sxx(:,:,j)'*W(:,:,j)*Sxx(:,:,j))*Sxx(:,:,j)'*W(:,:,j)*S(:,:,j)*W(:,:,j)*Sxx(:,:,j)*inv(Sxx(:,:,j)'*W(:,:,j)*Sxx(:,:,j));
    tstat(:,:,j)=(T(j)^(1/2))*b(:,:,j)./(diag(Avar(:,:,j)).^(1/2));
    SSRu(j)=e(:,1,j)'*e(:,1,j);
    Wald(j)=T(j)*(SSRr-SSRu(j))/SSRu(j);
    LM(j)=T(j)*(SSRr-SSRu(j))/SSRr;
    LR(j)=T(j)*log(SSRr/SSRu(j));
end
[supWald wald]=max(Wald);
[supLM lm]=max(LM);
[supLR lr]=max(LR);
Andrew = 11.72; % @ 5%, [0.15, 0.85]

resual = [1899 2012 b_r(1)/(1-b_r(2)) b_r(1) b_r(2) tstat_r(1) tstat_r(2) std_r supWald supLR supLM Andrew];
dlmwrite(strcat(rootpath,'Output\s012-13.structural_break_results.csv'),resual,"-append");

