function [w, EEP, ERe, ERf, CERe, Rf, G, Re, P, p] = mpep(mu,delta,rho,beta,alpha)
% Mehra Prescott (1985)'s code 
% INPUTS
% beta:     time-preference parameter
% alpha:    risk-aversion parameter
% mu:       unconditional expected consumption growth. See page 154 in
%           Mehra and Prescott (1985)
%           equivalent to rho0/(1-rho1) in the tauchen file 'equitypremiumTAUCHEN.m'.
% rho:      1st order serial correlation consumption growth. See page 154 in
%           Mehra and Prescott (1985)
%           equivalent to rho1 in the tauchen file 'equitypremiumTAUCHEN.m'.
% delta:    unconditional standard deviation consumption growth. See page 154 in
%           Mehra and Prescott (1985)
%           equivalent to s/sqrt(1-rho1^2) in the tauchen file 'equitypremiumTAUCHEN.m'.
% OUTPUT
% G:        column vector of states of consumption growth. See page 154 in
%           Mehra and Prescott (1985)
% P:        a matrix where P(i,j) is the probability of going from state
%           'i' to state 'j'. See eq. (4) in Mehra Prescott (1985).
% w:        equilibrium price-consumption ratio. See eq. (9) Mehra Prescott
%           (1985)
% Re:       a matrix where Re(i,j) is the return on the equity share from
%           state (c,i) to state (c*G(j),j). See eq. (10) in Mehra Prescott
%           (1985)
% CERe:     a vector where CERe(i) is the expected return on the equity
%           share conditional on state (c,i). See eq. (11)  in Mehra
%           Prescott (1985)
% pf:       a vector where pf(i) is the price of the risk-free security in
%           state (c,i). See eq. (12) in Mehra Prescott (1985)
% Rf:       a vector where Rf(i) is the return on the risk-free security
%           in state (c,i). See eq. (13) in Mehra Prescott (1985)
% p:        vector where p(i) is the stationary probability of state i. See
%           page 153 in Mehra Prescott (1985)
% ERe:      unconditional expected return on the equity share. See eq. (14)
%           in Mehra Prescott (1985)
% ERf:      unconditional expected return on the risk-free security. See
%           eq. (14) in Mehra Prescott (1985)
% EEP:      unconditional equity premium. See page 153 in Mehra Prescott
%           (1985)
% Standard parameters
if nargin == 0
    mu = 0.018;
    delta = 0.036;
    rho = -0.14;
    beta = 0.98;
    alpha = 2;
end
G = [1+mu+delta; 1+mu-delta];
N=length(G);
P = [(1+rho)/2 1-(1+rho)/2; 1-(1+rho)/2 (1+rho)/2];
for j=1:N
    for k=1:N
        A(j,k)=beta*(G(k)^(1-alpha))*P(j,k);
    end
end
w=inv(eye(N)-A)*A*ones(N,1); 
Re=zeros(N,N);
for j=1:N
    for k=1:N
        Re(j,k)=(((w(k)+1)/w(j))*G(k))-1; 
    end
end
for k=1:N
    CERe(k,1)=Re(k,:)*P(k,:)';
end
pf=P*beta*((G).^(-alpha));
Rf=1./pf-1;
p=ones(1,N)*inv(eye(N)-P+ones(N)); 
ERe=p*CERe;
ERf=p*Rf;
EEP=ERe-ERf;

